#!/usr/bin/ruby

require 'json'
require 'date'
require 'socket'
require 'syslog/logger'

$output = []

$time_now_sec = Time.now.to_i

$hostname = Socket.gethostname

ENV['PBSPRO_IGNORE_KERBEROS']='no'
ENV['PBS_AUTH_METHOD']='resvport'
ENV['PBS_ENCRYPT_METHOD']=''
path=ENV['PATH']
ENV['PATH'] = "#{path}:/usr/bin:/bin:/opt/pbs/bin"

$log = Syslog::Logger.new 'pbs_qstat.rb'

SUFFIX_MAG = { 'kb' => 1024, 'mb' => 1024*1024, 'gb' => 1024*1024*1024, 'tb' => 1024*1024*1024*1024 }

begin
  qstat=`qstat -Q`
rescue StandardError => msg
  $log.error("qstat error: #{msg}")
  exit 0
end

qstat.each_line do |line|
    next if line =~ /-----/
    next if line =~ /^Queue/
    next if line =~ /^M[0-9]+/

    line = line.gsub(/ [ ]*/, ' ')
    
    items = line.split(/ /)

    if items[3] == 'yes'
      if items[0] == 'default' or items[0] =~ /^q_/
        qtag = 'default'
      elsif items[0] =~ /gpu/
        qtag = 'gpu'
      else
        qtag = 'special'
      end

      $output << { 'name' => 'queued', 'qtag' => qtag, 'queue' => items[0], 'value' => items[5].to_i, 'time' => $time_now_sec, 'host' => $hostname }
      $output << { 'name' => 'running', 'qtag' => qtag, 'queue' => items[0], 'value' => items[6].to_i, 'time' => $time_now_sec, 'host' => $hostname }

      qinfo = JSON.parse(`qstat -Qf -F json #{items[0]}`)

      if qinfo['Queue'][items[0]]['resources_assigned'] != nil

        $output << { 'name' => 'queue_cpu', 'queue' => items[0], 
                     'value' => qinfo['Queue'][items[0]]['resources_assigned']['ncpus'].to_i,
                     'qtag' => qtag,
                     'time' => $time_now_sec, 'host' => $hostname }

        $output << { 'name' => 'queue_nodect', 'queue' => items[0],
                     'value' => qinfo['Queue'][items[0]]['resources_assigned']['nodect'].to_i,
                     'qtag' => qtag,
                     'time' => $time_now_sec, 'host' => $hostname }

        mem = qinfo['Queue'][items[0]]['resources_assigned']['mem']

        _, val, mag = (mem.match(/^([0-9]*)([a-z]*)/)).to_a

        mem = val.to_i
        mem = mem * SUFFIX_MAG[mag].to_i if mag != nil

        $output << { 'name' => 'queue_mem', 'queue' => items[0], 'qtag' => qtag,
                    'value' => mem, 'time' => $time_now_sec, 'host' => $hostname }
      end
     
    end
end

puts(JSON.pretty_generate($output))
