#!/usr/bin/ruby

require 'fileutils'
require 'json'
require 'open-uri'
require 'optparse'
require 'socket'
require 'syslog/logger'

$output = []

$owners = {
      'bee'    => 'cerit',
      'capy'   => 'cerit',
      'uruk'   => 'cerit',
      'ursa'   => 'cerit',
      'zelda'  => 'cerit',
      'zenon'  => 'cerit',
      'glados' => 'cerit',
      'gita'   => 'cerit',
      'zia'    => 'cerit',
      'elwe'   => 'elixir',
      'elmo'   => 'elixir',
      'eltu'   => 'elixir',
      'elan'   => 'elixir',
}

$hostname = Socket.gethostname

$time_now_sec = Time.now.to_i

$log = Syslog::Logger.new 'pbs_stats.rb'

use_file = nil
$affiliation_file = nil

OptionParser.new do |opts|
  opts.banner = "Usage: pbs_stats.rb [options]"

  opts.on("-fNAME", "--file=NAME", "Read log from file NAME") do |v|
    use_file = v
  end
  opts.on("-aNAME", "--affiliation-file=NAME", "Affiliation mapping location (fullpath with filename)") do |v|
    $affiliation_file = v
  end
end.parse!

exit 0 if $affiliation_file == nil

orgs = URI.open('https://metavo.metacentrum.cz/pbsmon2/api/org_names')

exit 0 if orgs == nil

begin
  orgs_a = JSON.parse(File.read(orgs.path).force_encoding('UTF-8'))
rescue StandardError => msg
  $log.error("JSON parse error: #{msg}")
  exit 0
end

orgs_h = {}

orgs_a.each do |org|
  orgs_h[org['user_org']] = org['name']
end

begin
  mappings = JSON.parse(File.read($affiliation_file).force_encoding('UTF-8'))
rescue StandardError => msg
  $log.error("affiliation file error: #{msg}")
  exit 0
end

af = {}

mappings.each do |item| 
   login = item['urn:perun:user_facility:attribute-def:virt:login']
   workplaceid = item['urn:perun:user:attribute-def:def:workplaceId']
   facultyid = item['urn:perun:user:attribute-def:def:enrolledFacultyID']
   deforg = item['urn:perun:user:attribute-def:def:organization']
   if item['urn:perun:user:attribute-def:virt:userOrganizations'] != nil
      metaorg = item['urn:perun:user:attribute-def:virt:userOrganizations']['meta']
      einfraorg = item['urn:perun:user:attribute-def:virt:userOrganizations']['einfra']
      elixirorg = item['urn:perun:user:attribute-def:virt:userOrganizations']['elixir-cz']
   end
   if metaorg != nil
      wplace = metaorg
   elsif einfraorg != nil
      wplace = einfraorg
   elsif elixirorg != nil
      wplace = elixirorg
   else
      wplace = deforg
   end

   if orgs_h[wplace] == nil
     $log.warn("unknown org: #{wplace}") if wplace != nil
     wplace = 'Others' 
   else
     wplace = orgs_h[wplace]
   end
   if login =~ /^9....../ 
      wplace = 'Guests'
   end
   if workplaceid != nil
      wplace = workplaceid
   elsif facultyid != nil
      wplace = facultyid
   end
   af[login] = wplace if af[login] == nil
end

date=(Time.now).strftime('%Y%m%d')

if use_file == nil
  log = File.read("/var/spool/pbs/server_priv/accounting/#{date}")
  if not File.file?('/var/tmp/lasttime')
    File.open('/var/tmp/lasttime', 'w').close
  end
  lasttime = (File.read("/var/tmp/lasttime").chomp).to_i
else
  log = File.read(use_file)
  lasttime = 0
end

tzone = (Time.now).strftime('%z')

total_cput=0
total_cpuwall=0
total_walltime=0

jobs = {}

time_used = {}

log.each_line do |line|
  items = line.split(';')  
  if items[1] == 'E'
    begin	 
      time_sec = DateTime.strptime(items[0], "%m/%d/%Y %H:%M:%S").strftime('%s')
    rescue
      next
    end      
    next if time_sec.to_i < lasttime
    lasttime = time_sec.to_i
    time_usec = time_sec.to_i * 1000000
    while time_used[time_usec] != nil
       time_usec += 1
    end
    time_used[time_usec] = 1
    jobid = items[2]
    params = items[3].split(' ')
    user = ''
    walltime = ''
    reqgpus = 0
    ncpus = ''
    queue = ''
    hosts = ''
    cput = ''
    mem = ''
    cpupercent = ''
    exit_status = ''
    jobname = ''
    ctime = 0
    start = 0
    reqmem = 0
    reqcpu = 0
    reqwalltime = 0
    scratch = 0
    gpumem = 0
    gpucap = ''
    params.each do |param|
      if param =~ /user=/
        user = param.gsub(/user=/, '')
      elsif param =~ /queue=/
        queue = param.gsub(/queue=/, '')
      elsif param =~ /resources_used.cput=/
        _tmp = param.gsub(/resources_used.cput=/, '').split(':')
        cput = (_tmp[0].to_i)*3600+(_tmp[1].to_i)*60+(_tmp[2].to_i) 
      elsif param =~ /resources_used.ncpus=/
        ncpus = param.gsub(/resources_used.ncpus=/, '')
      elsif param =~ /resources_used.mem=/
        mem = param.gsub(/resources_used.mem=/, '')
        mem = (mem.gsub(/kb/, '')).to_i * 1024
      elsif param =~ /resources_used.walltime=/
        _tmp = (param.gsub(/resources_used.walltime=/, '')).split(':')
        walltime = (_tmp[0].to_i)*3600+(_tmp[1].to_i)*60+(_tmp[2].to_i)
      elsif param =~ /resources_used.cpupercent=/
        cpupercent = param.gsub(/resources_used.cpupercent=/, '')
      elsif param =~ /exec_host=/
        _tmp = param.gsub(/exec_host=/,'')
        _tmp = _tmp.gsub(/\/[0-9\*]*/, '')
        _tmp = _tmp.split('+')
        hosts = _tmp.join(':')
      elsif param =~ /Exit_status=/
        exit_status = param.gsub(/Exit_status=/, '').to_i
      elsif param =~ /ctime=/
        ctime = param.gsub(/ctime=/, '').to_i
      elsif param =~ /start=/
        start = param.gsub(/start=/, '').to_i
      elsif param =~ /exec_vnode=/
        reqmem = (param.gsub(/.*mem=([0-9]*).*/, '\1')).to_i * 1024
	reqcpu = (param.gsub(/.*ncpus=([0-9]*).*/, '\1')).to_i
        if param =~ /ngpus=/
	   reqgpus = (param.gsub(/.*ngpus=([0-9]*).*/, '\1')).to_i
	end
      elsif param =~ /Resource_List.walltime=/
        _tmp = param.gsub(/Resource_List.walltime=/, '').split(':')
        reqwalltime = (_tmp[0].to_i)*3600+(_tmp[1].to_i)*60+(_tmp[2].to_i)
      elsif param =~ /jobname=/
        jobname = param.gsub(/jobname=/, '')
      elsif param =~ /Resource_List.select=/
	_tmp = param.gsub(/Resource_List.select=/, '').split(':')
	_tmp.each do |item|
	  if item =~ /gpu_cap=/
	    gpucap = item.gsub(/gpu_cap=/, '')
	  elsif item =~ /gpu_mem=/
	    gpumem = item.gsub(/gpu_mem=/, '')
            if gpumem =~ /kb/
	      gpumem = (gpumem.gsub(/kb/, '')).to_i * 1024
	    elsif gpumem =~ /mb/
              gpumem = (gpumem.gsub(/mb/, '')).to_i * 1024 * 1024
	    elsif gpumem =~ /gb/
	      gpumem = (gpumem.gsub(/gb/, '')).to_i * 1024 * 1024 * 1024
	    end
	  elsif item =~ /scratch.*=/
            scratch = item.gsub(/scratch.*=/, '')
            if scratch =~ /kb/
	      scratch = (scratch.gsub(/kb/, '')).to_i * 1024
	    elsif scratch =~ /mb/
              scratch = (scratch.gsub(/mb/, '')).to_i * 1024 * 1024
	    elsif scratch =~ /gb/
	      scratch = (scratch.gsub(/gb/, '')).to_i * 1024 * 1024 * 1024
	    end
	  end
	end
      end
    end
    if af[user]
      wid = af[user].gsub(/ /, '_')
    end
    wid = 'nil' if wid == nil
    
    if(ncpus.to_i != 0)
      cpupercent = (cpupercent.to_i/100.0)/ncpus.to_i
    else
      cpupercent = 0
    end

    if reqwalltime != 0
       walltimepercent = walltime.to_i*1.0 / reqwalltime
    else
       walltimepercent = 0
    end

    if reqmem != 0
       mempercent = mem.to_i * 1.0 / reqmem
    else
       mempercent = 0
    end

    next if reqcpu == 0

    node_o = hosts.gsub(/,.*/, '').gsub(/[0-9-]*/, '')

    if $owners[node_o] != nil
       node_owner = $owners[node_o]
    else
       node_owner = 'meta'
    end
     
    $output << {
           'name' => 'jobstats', 
           # values
           'cput' => walltime.to_i*ncpus.to_i,
           'walltime' => walltime.to_i,
           'walltimepercent' => walltimepercent,
           'cpu' => ncpus.to_i, 
	   'gpu' => reqgpus.to_i,
           'gput' => reqgpus.to_i*walltime.to_i,
	   'reqcpu' => reqcpu.to_i,
	   'reqgpu' => reqgpus.to_i,
           'reqmem' => reqmem.to_i,
           'cpupercent' => cpupercent,
           'memory' => mem.to_i, 
           'memoryt' => mem.to_i*walltime.to_i,
           'mempercent' => mempercent,
	   'gpumem' => gpumem,
	   'gpumemt' => gpumem*walltime.to_i,
	   'scratch' => scratch,
	   'scratcht' => scratch*walltime.to_i,
           # tags
           'user' => user, 
           'wid' => wid, 
           'queue' => queue, 
           'hosts' => hosts, 
           'host' => $hostname, 
           'ncpus' => ncpus, 
           'mem' => mem, 
           'exit_status' => exit_status, 
           'jobname' => jobname,
           'jobid' => jobid,
	   'gpucap' => gpucap,
           'owner' => node_owner,
           'time' => time_usec }
    time_usec = start*1000000
    while time_used[time_usec] != nil
       time_usec += 1
    end
    time_used[time_usec] = 1
    $output << {
           'name' => 'waittime', 
           'user' => user, 
           'wid' => wid, 
           'queue' => queue, 
           'hosts' => hosts, 
           'host' => $hostname, 
           'value' => start-ctime, 
           'ncpus' => sprintf("%03d",reqcpu.to_i), 
           'ngpus' => sprintf("%03d",reqgpus.to_i), 
           'mem' => sprintf("%014d",reqmem.to_i), 
	   'gpumem' => gpumem,
           'scratch' => scratch,
	   'gpucap' => gpucap,
           'jobname' => jobname,
           'jobid' => jobid,
           'owner' => node_owner,
           'time' => time_usec }
  end
end

puts(JSON.pretty_generate($output))

if use_file == nil
  f = File.new('/var/tmp/lasttime', 'wb', 0600)
  f.write(lasttime.to_s)
  f.close
end
